@@ -60,6 +60,7 @@ static int ipkg_info_cmd(ipkg_conf_t *conf, int argc, char **argv);
 static int ipkg_status_cmd(ipkg_conf_t *conf, int argc, char **argv);
 static int ipkg_install_pending_cmd(ipkg_conf_t *conf, int argc, char **argv);
 static int ipkg_install_cmd(ipkg_conf_t *conf, int argc, char **argv);
+static int ipkg_list_installed_cmd(ipkg_conf_t *conf, int argc, char **argv);
 static int ipkg_remove_cmd(ipkg_conf_t *conf, int argc, char **argv);
 static int ipkg_purge_cmd(ipkg_conf_t *conf, int argc, char **argv);
 static int ipkg_flag_cmd(ipkg_conf_t *conf, int argc, char **argv);
@@ -84,6 +85,7 @@ static ipkg_cmd_t cmds[] = {
      {"update", 0, ipkg_update_cmd}, 
      {"upgrade", 0, ipkg_upgrade_cmd},
      {"list", 0, ipkg_list_cmd},
+     {"list_installed", 0, ipkg_list_installed_cmd},
      {"info", 0, ipkg_info_cmd},
      {"flag", 1, ipkg_flag_cmd},
      {"status", 0, ipkg_status_cmd},
@@ -550,6 +552,54 @@ static int ipkg_list_cmd(ipkg_conf_t *conf, int argc, char **argv)
      return 0;
 }
 
+
+static int ipkg_list_installed_cmd(ipkg_conf_t *conf, int argc, char **argv)
+{
+     int i ;
+     pkg_vec_t *available;
+     pkg_t *pkg;
+     char desc_short[IPKG_LIST_DESCRIPTION_LENGTH];
+     char *newline;
+     char *pkg_name = NULL;
+     char *version_str;
+
+     if (argc > 0) {
+	  pkg_name = argv[0];
+     }
+     available = pkg_vec_alloc();
+     pkg_hash_fetch_all_installed(&conf->pkg_hash, available);
+     for (i=0; i < available->len; i++) {
+	  pkg = available->pkgs[i];
+	  /* if we have package name or pattern and pkg does not match, then skip it */
+	  if (pkg_name && fnmatch(pkg_name, pkg->name, 0)) 
+	       continue;
+	  if (pkg->description) {
+	       strncpy(desc_short, pkg->description, IPKG_LIST_DESCRIPTION_LENGTH);
+	  } else {
+	       desc_short[0] = '\0';
+	  }
+	  desc_short[IPKG_LIST_DESCRIPTION_LENGTH - 1] = '\0';
+	  newline = strchr(desc_short, '\n');
+	  if (newline) {
+	       *newline = '\0';
+	  }
+#ifndef IPKG_LIB
+	  printf("%s - %s\n", pkg->name, desc_short);
+#else
+	  if (ipkg_cb_list) {
+	  	version_str = pkg_version_str_alloc(pkg);
+	  	ipkg_cb_list(pkg->name,desc_short,
+		                             version_str,
+	                                 pkg->state_status,
+	                                 p_userdata);
+		free(version_str);
+	  }
+#endif
+     }
+
+     return 0;
+}
+
 static int ipkg_info_status_cmd(ipkg_conf_t *conf, int argc, char **argv, int installed_only)
 {
      int i;
